import * as React from 'react';
import { render } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import Input from './Input';

const changeHandler = jest.fn();

describe('<Input>', () => {
  it(`has a proper label for a11y`, () => {
    const { getByRole } = render(<Input id="id" name="name" label="Name" />);
    // This searches by a11y "name", labels provide names to inputs.
    const input = getByRole('textbox', { name: 'Name' });
    expect(input).toBeInTheDocument();
  });

  it(`can be set to value when uncontroller`, () => {
    const { getByRole } = render(<Input id="id" name="name" label="Name" />);
    const input = getByRole('textbox', { name: 'Name' });
    userEvent.type(input, 'Hello world!');
    expect(input).toHaveValue('Hello world!');
  });

  it(`can be set to value when controller`, () => {
    const { getByRole } = render(
      <Input
        id="id"
        name="name"
        label="Name"
        value=""
        onChange={changeHandler}
      />,
    );
    const input = getByRole('textbox', { name: 'Name' });
    userEvent.type(input, 'Hello world!');
    expect(changeHandler).toHaveBeenCalledTimes(12);
  });

  it(`can receive some extra classes`, () => {
    const { getByRole } = render(
      <Input classNames={['bad', 'robot']} id="id" name="name" label="Name" />,
    );
    const input = getByRole('textbox', { name: 'Name' });
    expect(input.parentElement).toHaveClass('bad');
    expect(input.parentElement).toHaveClass('robot');
  });

  it(`can be a required field`, () => {
    const { getByRole } = render(
      <Input required id="id" name="name" label="Name" />,
    );
    const input = getByRole('textbox', { name: 'Name' });
    expect(input.parentElement).toHaveClass('mandatory');
    expect(input).toHaveAttribute('required');
  });

  it(`can be a disabled field`, () => {
    const { getByRole } = render(
      <Input disabled id="id" name="name" label="Name" />,
    );
    const input = getByRole('textbox', { name: 'Name' });
    expect(input).toHaveAttribute('disabled');
    userEvent.type(input, "Hello world!");
    expect(input).toHaveValue('');
  });
  it.each([
    ['textarea', 'textbox'],
    ['email', 'textbox'],
    ['number', 'spinbutton'],
    ['checkbox', 'checkbox'],
  ])(`can be another type than text`, (type, role) => {
    const { getByRole } = render(
      <Input type={type} id="id" name="name" label="Name" />,
    );
    const input = getByRole(role, { name: 'Name' });
    expect(input).toBeInTheDocument();
  });
});
